#Daniel Ence November 9, 2009
#Short R function for use in the subsampler program.

if(FALSE){
###################Copied from ape library source code (Paradis E., Claude J. & Strimmer K. 2004. APE: analyses of phylogenetics and evolution in R language. Bioinformatics 20: 289–290)

read.tree <- function (file = "", text = NULL, tree.names = NULL, skip = 0, 
    comment.char = "#", keep.multi = FALSE, ...) 
{
    unname <- function(treetext) {
        nc <- nchar(treetext)
        tstart <- 1
        while (substr(treetext, tstart, tstart) != "(" && tstart <= 
            nc) tstart <- tstart + 1
        if (tstart > 1) 
            return(c(substr(treetext, 1, tstart - 1), substr(treetext, 
                tstart, nc)))
        return(c("", treetext))
    }
    if (!is.null(text)) {
        if (!is.character(text)) 
            stop("argument `text' must be of mode character")
        tree <- text
    }
    else {
        tree <- scan(file = file, what = "", sep = "\n", quiet = TRUE, 
            skip = skip, comment.char = comment.char, ...)
    }
    if (identical(tree, character(0))) {
        warning("empty character string.")
        return(NULL)
    }
    tree <- gsub("[ \t]", "", tree)
    tree <- unlist(strsplit(tree, NULL))
    y <- which(tree == ";")
    Ntree <- length(y)
    x <- c(1, y[-Ntree] + 1)
    if (is.na(y[1])) 
        return(NULL)
    STRING <- character(Ntree)
    for (i in 1:Ntree) STRING[i] <- paste(tree[x[i]:y[i]], sep = "", 
        collapse = "")
    tmp <- unlist(lapply(STRING, unname))
    tmpnames <- tmp[c(TRUE, FALSE)]
    STRING <- tmp[c(FALSE, TRUE)]
    if (is.null(tree.names) && any(nzchar(tmpnames))) 
        tree.names <- tmpnames
    colon <- grep(":", STRING)
    if (!length(colon)) {
        obj <- lapply(STRING, clado.build)
    }
    else if (length(colon) == Ntree) {
        obj <- lapply(STRING, tree.build)
    }
    else {
        obj <- vector("list", Ntree)
        obj[colon] <- lapply(STRING[colon], tree.build)
        nocolon <- (1:Ntree)[!1:Ntree %in% colon]
        obj[nocolon] <- lapply(STRING[nocolon], clado.build)
    }
    for (i in 1:Ntree) {
        ROOT <- length(obj[[i]]$tip.label) + 1
        if (sum(obj[[i]]$edge[, 1] == ROOT) == 1 && dim(obj[[i]]$edge)[1] > 
            1) 
            stop(paste("There is apparently two root edges in your file: cannot read tree file.\n  Reading Newick file aborted at tree no.", 
                i, sep = ""))
    }
    if (Ntree == 1 && !keep.multi) 
        obj <- obj[[1]]
    else {
        if (!is.null(tree.names)) 
            names(obj) <- tree.names
        class(obj) <- "multiPhylo"
    }
    obj
}



write.tree <- function (phy, file = "", append = FALSE, digits = 10, tree.names = FALSE) 
{
    output.tree.names <- FALSE
    if (is.logical(tree.names)) {
        output.tree.names <- tree.names
        tree.names <- NULL
    }
    else if (is.character(tree.names)) {
        output.tree.names <- TRUE
        names(phy) <- tree.names
    }
    if (output.tree.names) 
        names(phy) <- checkLabel(names(phy))
    if (inherits(phy, "multiPhylo")) {
        write.tree(phy[[1]], file = file, append = append, digits = digits, 
            tree.names = names(phy)[1])
        if (length(phy) > 1) 
            for (i in 2:length(phy)) write.tree(phy[[i]], file = file, 
                append = TRUE, digits = digits, tree.names = names(phy)[i])
        return(invisible(NULL))
    }
    if (!inherits(phy, "phylo")) 
        stop("object \"phy\" is not of class \"phylo\"")
    brl <- !is.null(phy$edge.length)
    nodelab <- !is.null(phy$node.label)
    phy$tip.label <- checkLabel(phy$tip.label)
    if (nodelab) 
        phy$node.label <- checkLabel(phy$node.label)
    f.d <- paste("%.", digits, "g", sep = "")
    cp <- function(s) STRING <<- paste(STRING, s, sep = "")
    add.internal <- function(i) {
        cp("(")
        br <- which(phy$edge[, 1] == i)
        for (j in br) {
            desc <- phy$edge[j, 2]
            if (desc > n) 
                add.internal(desc)
            else add.terminal(j)
            if (j != br[length(br)]) 
                cp(",")
        }
        cp(")")
        if (nodelab) 
            cp(phy$node.label[i - n])
        if (brl) {
            cp(":")
            cp(sprintf(f.d, phy$edge.length[which(phy$edge[, 
                2] == i)]))
        }
    }
    add.terminal <- function(i) {
        cp(phy$tip.label[phy$edge[i, 2]])
        if (brl) {
            cp(":")
            cp(sprintf(f.d, phy$edge.length[i]))
        }
    }
    n <- length(phy$tip.label)
    STRING <- if (output.tree.names) 
        paste(tree.names, "(", sep = "")
    else "("
    br <- which(phy$edge[, 1] == n + 1)
    for (j in br) {
        desc <- phy$edge[j, 2]
        if (desc > n) 
            add.internal(desc)
        else add.terminal(j)
        if (j != br[length(br)]) 
            cp(",")
    }
    if (is.null(phy$root.edge)) {
        cp(")")
        if (nodelab) 
            cp(phy$node.label[1])
        cp(";")
    }
    else {
        cp(")")
        if (nodelab) 
            cp(phy$node.label[1])
        cp(":")
        cp(sprintf(f.d, phy$root.edge))
        cp(";")
    }
    if (file == "") 
        return(STRING)
    else cat(STRING, file = file, append = append, sep = "\n")
}


is.binary.tree <- function (phy) 
{
    if (!inherits(phy, "phylo")) 
        stop("object \"phy\" is not of class \"phylo\"")
    nb.tip <- length(phy$tip.label)
    nb.node <- phy$Nnode
    if (is.rooted(phy)) {
        if (nb.tip - 1 == nb.node) 
            return(TRUE)
        else return(FALSE)
    }
    else {
        if (nb.tip - 2 == nb.node) 
            return(TRUE)
        else return(FALSE)
    }
}





multi2di <- function (phy, random = TRUE) 
{
    degree <- tabulate(phy$edge[, 1])
    target <- which(degree > 2)
    if (!length(target)) 
        return(phy)
    nb.edge <- dim(phy$edge)[1]
    nextnode <- length(phy$tip.label) + phy$Nnode + 1
    new.edge <- edge2delete <- NULL
    wbl <- FALSE
    if (!is.null(phy$edge.length)) {
        wbl <- TRUE
        new.edge.length <- NULL
    }
    for (node in target) {
        ind <- which(phy$edge[, 1] == node)
        N <- length(ind)
        desc <- phy$edge[ind, 2]
        if (random) {
            tmp <- sample(length(desc))
            desc <- desc[tmp]
            res <- rtree(N)$edge
        }
        else {
            res <- matrix(0, 2 * N - 2, 2)
            res[, 1] <- N + rep(1:(N - 1), each = 2)
            res[, 2] <- N + rep(2:N, each = 2)
            res[seq(1, by = 2, length.out = N - 1), 2] <- 1:(N - 
                1)
            res[length(res)] <- N
        }
        if (wbl) {
            el <- numeric(dim(res)[1])
            el[res[, 2] <= N] <- if (random) 
                phy$edge.length[ind][tmp]
            else phy$edge.length[ind]
        }
        Nodes <- c(node, seq(from = nextnode, length.out = N - 
            2))
        res[, 1] <- Nodes[res[, 1] - N]
        tmp <- res[, 2] > N
        res[tmp, 2] <- Nodes[res[tmp, 2] - N]
        res[!tmp, 2] <- desc[res[!tmp, 2]]
        new.edge <- rbind(new.edge, res)
        edge2delete <- c(edge2delete, ind)
        if (wbl) 
            new.edge.length <- c(new.edge.length, el)
        nextnode <- nextnode + N - 2
        phy$Nnode <- phy$Nnode + N - 2
    }
    phy$edge <- rbind(phy$edge[-edge2delete, ], new.edge)
    if (wbl) 
        phy$edge.length <- c(phy$edge.length[-edge2delete], new.edge.length)
    if (!is.null(attr(phy, "order"))) 
        attr(phy, "order") <- NULL
    if (!is.null(phy$node.label)) 
        phy$node.label <- c(phy$node.label, rep("", phy$Nnode - 
            length(phy$node.label)))
	phy
}

tree.build <- function (tp) 
{
    add.internal <- function() {
        edge[j, 1] <<- current.node
        edge[j, 2] <<- current.node <<- node <<- node + 1L
        j <<- j + 1L
    }
    add.terminal <- function() {
        edge[j, 1] <<- current.node
        edge[j, 2] <<- tip
        X <- unlist(strsplit(tpc[k], ":"))
        tip.label[tip] <<- X[1]
        edge.length[j] <<- as.numeric(X[2])
        k <<- k + 1L
        tip <<- tip + 1L
        j <<- j + 1L
    }
    go.down <- function() {
        l <- which(edge[, 2] == current.node)
        X <- unlist(strsplit(tpc[k], ":"))
        node.label[current.node - nb.tip] <<- X[1]
        edge.length[l] <<- as.numeric(X[2])
        k <<- k + 1L
        current.node <<- edge[l, 1]
    }
    if (!length(grep(",", tp))) {
        obj <- list(edge = matrix(c(2L, 1L), 1, 2))
        tp <- unlist(strsplit(tp, "[\\(\\):;]"))
        obj$edge.length <- as.numeric(tp[3])
        obj$Nnode <- 1L
        obj$tip.label <- tp[2]
        if (length(tp) == 4) 
            obj$node.label <- tp[4]
        class(obj) <- "phylo"
        return(obj)
    }
    tpc <- unlist(strsplit(tp, "[\\(\\),;]"))
    tpc <- tpc[nzchar(tpc)]
    tsp <- unlist(strsplit(tp, NULL))
    skeleton <- tsp[tsp %in% c("(", ")", ",", ";")]
    nsk <- length(skeleton)
    nb.node <- sum(skeleton == ")")
    nb.tip <- sum(skeleton == ",") + 1
    nb.edge <- nb.node + nb.tip
    node.label <- character(nb.node)
    tip.label <- character(nb.tip)
    edge.length <- numeric(nb.edge)
    edge <- matrix(0L, nb.edge, 2)
    current.node <- node <- as.integer(nb.tip + 1)
    edge[nb.edge, 2] <- node
    j <- k <- tip <- 1L
    for (i in 2:nsk) {
        if (skeleton[i] == "(") 
            add.internal()
        if (skeleton[i] == ",") {
            if (skeleton[i - 1] != ")") 
                add.terminal()
        }
        if (skeleton[i] == ")") {
            if (skeleton[i - 1] == ",") {
                add.terminal()
                go.down()
            }
            if (skeleton[i - 1] == ")") 
                go.down()
        }
    }
    edge <- edge[-nb.edge, ]
    obj <- list(edge = edge, Nnode = nb.node, tip.label = tip.label)
    root.edge <- edge.length[nb.edge]
    edge.length <- edge.length[-nb.edge]
    if (!all(is.na(edge.length))) 
        obj$edge.length <- edge.length
    if (is.na(node.label[1])) 
        node.label[1] <- ""
    if (any(nzchar(node.label))) 
        obj$node.label <- node.label
    if (!is.na(root.edge)) 
        obj$root.edge <- root.edge
    class(obj) <- "phylo"
    obj
}


#if(FALSE)
{
	library(gee)
	library(ape)	
}



chronogram <- function (phy, scale = 1, expo = 2, minEdgeLength = 1e-06) 
{
    if (class(phy) != "phylo") 
        stop("object \"phy\" is not of class \"phylo\"")
    phy <- new2old.phylo(phy)
    prepareTree(phy, minEdgeLength = minEdgeLength)
    opt <- optimTree(phy, expo = expo)
    newTree <- phy
    newTree$edge.length <- getDurations(opt$par, scale)
    ans <- newTree
    old2new.phylo(ans)
}



getDurations <- function (params, scale) 
.C("getDurations", as.double(params), as.double(scale), result = double(getNEdges()), 
    PACKAGE = "ape")$result


checkLabel <- function (x, ...) 
{
    x <- gsub("^[[:space:]\\(]+", "", x)
    x <- gsub("[[:space:]\\)]+$", "", x)
    x <- gsub("[[:space:]]", "_", x)
    x <- gsub("[,:;]", "", x)
    x <- gsub("[\\(\\)]", "-", x)
    x <- gsub("_{2,}", "_", x)
    x <- gsub("-{2,}", "-", x)
    x
}


is.rooted <- function (phy) 
{
    if (!inherits(phy, "phylo")) 
        stop("object \"phy\" is not of class \"phylo\"")
    if (!is.null(phy$root.edge)) 
        TRUE
    else if (tabulate(phy$edge[, 1])[length(phy$tip.label) + 
        1] > 2) 
        FALSE
    else TRUE
}








rtree <- function (n, rooted = TRUE, tip.label = NULL, br = runif, ...) 
{
    foo <- function(n, pos) {
        n1 <- .Internal(sample(n - 1, 1, FALSE, NULL))
        n2 <- n - n1
        po2 <- pos + 2 * n1 - 1
        edge[c(pos, po2), 1] <<- nod
        nod <<- nod + 1L
        if (n1 > 2) {
            edge[pos, 2] <<- nod
            foo(n1, pos + 1)
        }
        else if (n1 == 2) {
            edge[c(pos + 1, pos + 2), 1] <<- edge[pos, 2] <<- nod
            nod <<- nod + 1L
        }
        if (n2 > 2) {
            edge[po2, 2] <<- nod
            foo(n2, po2 + 1)
        }
        else if (n2 == 2) {
            edge[c(po2 + 1, po2 + 2), 1] <<- edge[po2, 2] <<- nod
            nod <<- nod + 1L
        }
    }
    if (n < 2) 
        stop("a tree must have at least 2 tips.")
    nbr <- 2 * n - 3 + rooted
    edge <- matrix(NA, nbr, 2)
    n <- as.integer(n)
    if (n == 2) {
        if (rooted) 
            edge[] <- c(3L, 3L, 1L, 2L)
        else stop("an unrooted tree must have at least 3 tips.")
    }
    else if (n == 3) {
        edge[] <- if (rooted) 
            c(4L, 5L, 5L, 4L, 5L, 1:3)
        else c(4L, 4L, 4L, 1:3)
    }
    else if (n == 4 && !rooted) {
        edge[] <- c(5L, 6L, 6L, 5L, 5L, 6L, 1:4)
    }
    else {
        nod <- n + 1L
        if (rooted) {
            foo(n, 1)
            i <- which(is.na(edge[, 2]))
            edge[i, 2] <- 1:n
        }
        else {
            n1 <- .Internal(sample(n - 2, 1, FALSE, NULL))
            if (n1 == n - 2) {
                n2 <- n3 <- 1
            }
            else {
                n2 <- .Internal(sample(n - n1 - 1, 1, FALSE, 
                  NULL))
                n3 <- n - n1 - n2
            }
            po2 <- 2 * n1
            po3 <- 2 * (n1 + n2) - 1
            edge[c(1, po2, po3), 1] <- nod
            nod <- nod + 1
            if (n1 > 2) {
                edge[1, 2] <- nod
                foo(n1, 2)
            }
            else if (n1 == 2) {
                edge[2:3, 1] <- edge[1, 2] <- nod
                nod <- nod + 1L
            }
            if (n2 > 2) {
                edge[po2, 2] <- nod
                foo(n2, po2 + 1)
            }
            else if (n2 == 2) {
                edge[c(po2 + 1, po2 + 2), 1] <- edge[po2, 2] <- nod
                nod <- nod + 1L
            }
            if (n3 > 2) {
                edge[po3, 2] <- nod
                foo(n3, po3 + 1)
            }
            else if (n3 == 2) {
                edge[c(po3 + 1, po3 + 2), 1] <- edge[po3, 2] <- nod
            }
            i <- which(is.na(edge[, 2]))
            edge[i, 2] <- 1:n
        }
    }
    phy <- list(edge = edge)
    phy$tip.label <- if (is.null(tip.label)) 
        paste("t", sample(n), sep = "")
    else sample(tip.label)
    if (is.function(br)) 
        phy$edge.length <- br(nbr, ...)
    phy$Nnode <- n - 2L + rooted
    class(phy) <- "phylo"
    phy
}



unroot <- function (phy) 
{
    if (!inherits(phy, "phylo")) 
        stop("object \"phy\" is not of class \"phylo\"")
    if (dim(phy$edge)[1] < 3) 
        stop("cannot unroot a tree with two edges.")
    if (!is.null(phy$root.edge)) 
        phy$root.edge <- NULL
    if (!is.rooted(phy)) 
        return(phy)
    nb.tip <- length(phy$tip.label)
    ROOT <- nb.tip + 1
    EDGEROOT <- which(phy$edge[, 1] == ROOT)
    if (phy$edge[EDGEROOT[1], 2] == ROOT + 1) {
        j <- EDGEROOT[2]
        i <- EDGEROOT[1]
    }
    else {
        j <- EDGEROOT[1]
        i <- EDGEROOT[2]
    }
    phy$edge <- phy$edge[-i, ]
    nodes <- phy$edge > ROOT
    phy$edge[nodes] <- phy$edge[nodes] - 1
    if (!is.null(phy$edge.length)) {
        phy$edge.length[j] <- phy$edge.length[j] + phy$edge.length[i]
        phy$edge.length <- phy$edge.length[-i]
    }
    phy$Nnode <- phy$Nnode - 1
    if (!is.null(phy$node.label)) 
        phy$node.label <- phy$node.label[-2]
    phy
}

##########End of ape code.
}






library(gee)
library(ape)


if(FALSE)
{
midPointRoot <- function(phy)
{
	if (!inherits(phy, "phylo")) 
        stop("object \"phy\" is not of class \"phylo\"")
    		
}
}







setwd
getwd()

myTree <- read.tree(file = "filename")
cat("Original tree is: ", write.tree(myTree), "\n")

if(FALSE)
{
if(is.rooted(myTree))
{
	myTree <- unroot(myTree)
}
}





#myTree <- chronogram(myTree)
#myTree <- ratogram(myTree)
#myTree <- chronopl(myTree, 1000000, tol = 1e-20)

while(!is.binary.tree(myTree))
{
	myTree <- multi2di(myTree)
}
myTree <- chronoMPL(myTree, FALSE, FALSE)
write.tree(myTree, file = "filename")	
cat("Fixed tree is: ", write.tree(myTree), "\n")
